package sun.util.locale;

import java.util.HashMap;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Map;
import java.util.List;

public class LanguageTag
{
    public static final String SEP = "-";
    public static final String PRIVATEUSE = "x";
    public static final String UNDETERMINED = "und";
    public static final String PRIVUSE_VARIANT_PREFIX = "lvariant";
    private String language;
    private String script;
    private String region;
    private String privateuse;
    private List<String> extlangs;
    private List<String> variants;
    private List<String> extensions;
    private static final Map<String, String[]> GRANDFATHERED;
    
    private LanguageTag() {
        this.language = "";
        this.script = "";
        this.region = "";
        this.privateuse = "";
        this.extlangs = Collections.emptyList();
        this.variants = Collections.emptyList();
        this.extensions = Collections.emptyList();
    }
    
    public static LanguageTag parse(String s, ParseStatus parseStatus) {
        if (parseStatus == null) {
            parseStatus = new ParseStatus();
        }
        else {
            parseStatus.reset();
        }
        String[] array = (String[])LanguageTag.GRANDFATHERED.get(LocaleUtils.toLowerString(s));
        Object o;
        if (array != null) {
            o = new StringTokenIterator(array[1], "-");
        }
        else {
            o = new StringTokenIterator(s, "-");
        }
        LanguageTag o2 = new LanguageTag();
        if (o2.parseLanguage(o, parseStatus)) {
            o2.parseExtlangs(o, parseStatus);
            o2.parseScript(o, parseStatus);
            o2.parseRegion(o, parseStatus);
            o2.parseVariants(o, parseStatus);
            o2.parseExtensions(o, parseStatus);
        }
        o2.parsePrivateuse(o, parseStatus);
        if (!o.isDone() && !parseStatus.isError()) {
            String current = o.current();
            parseStatus.errorIndex = o.currentStart();
            if (current.length() == 0) {
                parseStatus.errorMsg = "Empty subtag";
            }
            else {
                parseStatus.errorMsg = "Invalid subtag: " + current;
            }
        }
        return o2;
    }
    
    private boolean parseLanguage(StringTokenIterator stringTokenIterator, ParseStatus parseStatus) {
        if (stringTokenIterator.isDone() || parseStatus.isError()) {
            return false;
        }
        boolean n = false;
        String current = stringTokenIterator.current();
        if (isLanguage(current)) {
            n = true;
            this.language = current;
            parseStatus.parseLength = stringTokenIterator.currentEnd();
            stringTokenIterator.next();
        }
        return n;
    }
    
    private boolean parseExtlangs(StringTokenIterator stringTokenIterator, ParseStatus parseStatus) {
        if (stringTokenIterator.isDone() || parseStatus.isError()) {
            return false;
        }
        boolean n = false;
        while (!stringTokenIterator.isDone()) {
            String current = stringTokenIterator.current();
            if (!isExtlang(current)) {
                break;
            }
            n = true;
            if (this.extlangs.isEmpty()) {
                this.extlangs = new ArrayList(3);
            }
            this.extlangs.add(current);
            parseStatus.parseLength = stringTokenIterator.currentEnd();
            stringTokenIterator.next();
            if (this.extlangs.size() == 3) {
                break;
            }
            continue;
        }
        return n;
    }
    
    private boolean parseScript(StringTokenIterator stringTokenIterator, ParseStatus parseStatus) {
        if (stringTokenIterator.isDone() || parseStatus.isError()) {
            return false;
        }
        boolean n = false;
        String current = stringTokenIterator.current();
        if (isScript(current)) {
            n = true;
            this.script = current;
            parseStatus.parseLength = stringTokenIterator.currentEnd();
            stringTokenIterator.next();
        }
        return n;
    }
    
    private boolean parseRegion(StringTokenIterator stringTokenIterator, ParseStatus parseStatus) {
        if (stringTokenIterator.isDone() || parseStatus.isError()) {
            return false;
        }
        boolean n = false;
        String current = stringTokenIterator.current();
        if (isRegion(current)) {
            n = true;
            this.region = current;
            parseStatus.parseLength = stringTokenIterator.currentEnd();
            stringTokenIterator.next();
        }
        return n;
    }
    
    private boolean parseVariants(StringTokenIterator stringTokenIterator, ParseStatus parseStatus) {
        if (stringTokenIterator.isDone() || parseStatus.isError()) {
            return false;
        }
        boolean n = false;
        while (!stringTokenIterator.isDone()) {
            String current = stringTokenIterator.current();
            if (!isVariant(current)) {
                break;
            }
            n = true;
            if (this.variants.isEmpty()) {
                this.variants = new ArrayList(3);
            }
            this.variants.add(current);
            parseStatus.parseLength = stringTokenIterator.currentEnd();
            stringTokenIterator.next();
        }
        return n;
    }
    
    private boolean parseExtensions(StringTokenIterator stringTokenIterator, ParseStatus parseStatus) {
        if (stringTokenIterator.isDone() || parseStatus.isError()) {
            return false;
        }
        boolean n = false;
        while (!stringTokenIterator.isDone()) {
            String current = stringTokenIterator.current();
            if (isExtensionSingleton(current)) {
                int currentStart = stringTokenIterator.currentStart();
                StringBuilder o = new StringBuilder(current);
                stringTokenIterator.next();
                while (!stringTokenIterator.isDone()) {
                    String current2 = stringTokenIterator.current();
                    if (isExtensionSubtag(current2)) {
                        o.append("-").append(current2);
                        parseStatus.parseLength = stringTokenIterator.currentEnd();
                        stringTokenIterator.next();
                    }
                    else {
                        break;
                    }
                }
                if (parseStatus.parseLength <= currentStart) {
                    parseStatus.errorIndex = currentStart;
                    parseStatus.errorMsg = "Incomplete extension '" + current + "'";
                    break;
                }
                if (this.extensions.isEmpty()) {
                    this.extensions = new ArrayList(4);
                }
                this.extensions.add(o.toString());
                n = true;
            }
            else {
                break;
            }
        }
        return n;
    }
    
    private boolean parsePrivateuse(StringTokenIterator stringTokenIterator, ParseStatus parseStatus) {
        if (stringTokenIterator.isDone() || parseStatus.isError()) {
            return false;
        }
        boolean n = false;
        String current = stringTokenIterator.current();
        if (isPrivateusePrefix(current)) {
            int currentStart = stringTokenIterator.currentStart();
            StringBuilder sb = new StringBuilder(current);
            stringTokenIterator.next();
            while (!stringTokenIterator.isDone()) {
                String current2 = stringTokenIterator.current();
                if (!isPrivateuseSubtag(current2)) {
                    break;
                }
                sb.append("-").append(current2);
                parseStatus.parseLength = stringTokenIterator.currentEnd();
                stringTokenIterator.next();
            }
            if (parseStatus.parseLength <= currentStart) {
                parseStatus.errorIndex = currentStart;
                parseStatus.errorMsg = "Incomplete privateuse";
            }
            else {
                this.privateuse = sb.toString();
                n = true;
            }
        }
        return n;
    }
    
    public static LanguageTag parseLocale(BaseLocale baseLocale, LocaleExtensions localeExtensions) {
        LanguageTag languageTag = new LanguageTag();
        String language = baseLocale.getLanguage();
        String script = baseLocale.getScript();
        String region = baseLocale.getRegion();
        String variant = baseLocale.getVariant();
        boolean n = false;
        String string = null;
        if (isLanguage(language)) {
            if (language.equals("iw")) {
                language = "he";
            }
            else if (language.equals("ji")) {
                language = "yi";
            }
            else if (language.equals("in")) {
                language = "id";
            }
            languageTag.language = language;
        }
        if (isScript(script)) {
            languageTag.script = canonicalizeScript(script);
            n = true;
        }
        if (isRegion(region)) {
            languageTag.region = canonicalizeRegion(region);
            n = true;
        }
        if (languageTag.language.equals("no") && languageTag.region.equals("NO") && variant.equals("NY")) {
            languageTag.language = "nn";
            variant = "";
        }
        if (variant.length() > 0) {
            Object variants = null;
            StringTokenIterator o = new StringTokenIterator(variant, "_");
            while (!o.isDone()) {
                String current = o.current();
                if (!isVariant(current)) {
                    break;
                }
                if (variants == null) {
                    variants = new ArrayList();
                }
                variants.add(current);
                o.next();
            }
            if (variants != null) {
                languageTag.variants = variants;
                n = true;
            }
            if (!o.isDone()) {
                StringBuilder o2 = new StringBuilder();
                while (!o.isDone()) {
                    String current2 = o.current();
                    if (!isPrivateuseSubtag(current2)) {
                        break;
                    }
                    if (o2.length() > 0) {
                        o2.append("-");
                    }
                    o2.append(current2);
                    o.next();
                }
                if (o2.length() > 0) {
                    string = o2.toString();
                }
            }
        }
        Object extensions = null;
        String s = null;
        if (localeExtensions != null) {
            for (Character c : localeExtensions.getKeys()) {
                Extension extension = localeExtensions.getExtension(c);
                if (isPrivateusePrefixChar((char)c)) {
                    s = extension.getValue();
                }
                else {
                    if (extensions == null) {
                        extensions = new ArrayList();
                    }
                    extensions.add(c.toString() + "-" + extension.getValue());
                }
            }
        }
        if (extensions != null) {
            languageTag.extensions = extensions;
            n = true;
        }
        if (string != null) {
            if (s == null) {
                s = "lvariant-" + string;
            }
            else {
                s = s + "-" + "lvariant" + "-" + string.replace("_", "-");
            }
        }
        if (s != null) {
            languageTag.privateuse = s;
        }
        if (languageTag.language.length() == 0 && (n || s == null)) {
            languageTag.language = "und";
            return languageTag;
        }
        return languageTag;
    }
    
    public String getLanguage() {
        return this.language;
    }
    
    public List<String> getExtlangs() {
        if (this.extlangs.isEmpty()) {
            return Collections.emptyList();
        }
        return Collections.unmodifiableList(this.extlangs);
    }
    
    public String getScript() {
        return this.script;
    }
    
    public String getRegion() {
        return this.region;
    }
    
    public List<String> getVariants() {
        if (this.variants.isEmpty()) {
            return Collections.emptyList();
        }
        return Collections.unmodifiableList(this.variants);
    }
    
    public List<String> getExtensions() {
        if (this.extensions.isEmpty()) {
            return Collections.emptyList();
        }
        return Collections.unmodifiableList(this.extensions);
    }
    
    public String getPrivateuse() {
        return this.privateuse;
    }
    
    public static boolean isLanguage(String s) {
        int length = s.length();
        return length >= 2 && length <= 8 && LocaleUtils.isAlphaString(s);
    }
    
    public static boolean isExtlang(String s) {
        return s.length() == 3 && LocaleUtils.isAlphaString(s);
    }
    
    public static boolean isScript(String s) {
        return s.length() == 4 && LocaleUtils.isAlphaString(s);
    }
    
    public static boolean isRegion(String s) {
        return (s.length() == 2 && LocaleUtils.isAlphaString(s)) || (s.length() == 3 && LocaleUtils.isNumericString(s));
    }
    
    public static boolean isVariant(String s) {
        int length = s.length();
        if (length >= 5 && length <= 8) {
            return LocaleUtils.isAlphaNumericString(s);
        }
        return length == 4 && LocaleUtils.isNumeric(s.charAt(0)) && LocaleUtils.isAlphaNumeric(s.charAt(1)) && LocaleUtils.isAlphaNumeric(s.charAt(2)) && LocaleUtils.isAlphaNumeric(s.charAt(3));
    }
    
    public static boolean isExtensionSingleton(String s) {
        return s.length() == 1 && LocaleUtils.isAlphaString(s) && !LocaleUtils.caseIgnoreMatch("x", s);
    }
    
    public static boolean isExtensionSingletonChar(char c) {
        return isExtensionSingleton(String.valueOf(c));
    }
    
    public static boolean isExtensionSubtag(String s) {
        int length = s.length();
        return length >= 2 && length <= 8 && LocaleUtils.isAlphaNumericString(s);
    }
    
    public static boolean isPrivateusePrefix(String s) {
        return s.length() == 1 && LocaleUtils.caseIgnoreMatch("x", s);
    }
    
    public static boolean isPrivateusePrefixChar(char c) {
        return LocaleUtils.caseIgnoreMatch("x", String.valueOf(c));
    }
    
    public static boolean isPrivateuseSubtag(String s) {
        int length = s.length();
        return length >= 1 && length <= 8 && LocaleUtils.isAlphaNumericString(s);
    }
    
    public static String canonicalizeLanguage(String s) {
        return LocaleUtils.toLowerString(s);
    }
    
    public static String canonicalizeExtlang(String s) {
        return LocaleUtils.toLowerString(s);
    }
    
    public static String canonicalizeScript(String s) {
        return LocaleUtils.toTitleString(s);
    }
    
    public static String canonicalizeRegion(String s) {
        return LocaleUtils.toUpperString(s);
    }
    
    public static String canonicalizeVariant(String s) {
        return LocaleUtils.toLowerString(s);
    }
    
    public static String canonicalizeExtension(String s) {
        return LocaleUtils.toLowerString(s);
    }
    
    public static String canonicalizeExtensionSingleton(String s) {
        return LocaleUtils.toLowerString(s);
    }
    
    public static String canonicalizeExtensionSubtag(String s) {
        return LocaleUtils.toLowerString(s);
    }
    
    public static String canonicalizePrivateuse(String s) {
        return LocaleUtils.toLowerString(s);
    }
    
    public static String canonicalizePrivateuseSubtag(String s) {
        return LocaleUtils.toLowerString(s);
    }
    
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        if (this.language.length() > 0) {
            sb.append(this.language);
            Iterator<String> iterator = this.extlangs.iterator();
            while (iterator.hasNext()) {
                sb.append("-").append((String)iterator.next());
            }
            if (this.script.length() > 0) {
                sb.append("-").append(this.script);
            }
            if (this.region.length() > 0) {
                sb.append("-").append(this.region);
            }
            Iterator<String> iterator2 = this.variants.iterator();
            while (iterator2.hasNext()) {
                sb.append("-").append((String)iterator2.next());
            }
            Iterator<String> iterator3 = this.extensions.iterator();
            while (iterator3.hasNext()) {
                sb.append("-").append((String)iterator3.next());
            }
        }
        if (this.privateuse.length() > 0) {
            if (sb.length() > 0) {
                sb.append("-");
            }
            sb.append(this.privateuse);
        }
        return sb.toString();
    }
    
    static {
        GRANDFATHERED = new HashMap();
        String[][] array;
        for (String[] array2 : array = new String[][] { { "art-lojban", "jbo" }, { "cel-gaulish", "xtg-x-cel-gaulish" }, { "en-GB-oed", "en-GB-x-oed" }, { "i-ami", "ami" }, { "i-bnn", "bnn" }, { "i-default", "en-x-i-default" }, { "i-enochian", "und-x-i-enochian" }, { "i-hak", "hak" }, { "i-klingon", "tlh" }, { "i-lux", "lb" }, { "i-mingo", "see-x-i-mingo" }, { "i-navajo", "nv" }, { "i-pwn", "pwn" }, { "i-tao", "tao" }, { "i-tay", "tay" }, { "i-tsu", "tsu" }, { "no-bok", "nb" }, { "no-nyn", "nn" }, { "sgn-BE-FR", "sfb" }, { "sgn-BE-NL", "vgt" }, { "sgn-CH-DE", "sgg" }, { "zh-guoyu", "cmn" }, { "zh-hakka", "hak" }, { "zh-min", "nan-x-zh-min" }, { "zh-min-nan", "nan" }, { "zh-xiang", "hsn" } }) {
            LanguageTag.GRANDFATHERED.put(LocaleUtils.toLowerString(array2[0]), array2);
        }
    }
}
